/*
** Module   : ChuiWebSimulator.java
** Abstract : A WebSocket Chui Terminal simulator.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20131113 Created initial version
*/

package com.goldencode.p2j.ui.client.chui.driver.web;

import java.io.*;
import java.nio.*;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.jetty.websocket.api.*;
import org.eclipse.jetty.websocket.api.annotations.*;

import com.goldencode.p2j.ui.client.chui.driver.swing.*;
import com.goldencode.p2j.util.*;

/**
 *  Chui WebSocket terminal simulator.
 *
 */
@SuppressWarnings("serial")
@WebSocket
public class ChuiWebSimulator
extends ChuiSimulator
{  
   /** Logger. */
   private static final Logger LOG = LogHelper.getLogger(ChuiWebSimulator.class.getName());   
   
   /**  Object representing the other end of this conversation. */
   private RemoteEndpoint remote;  
   
   /** Keyboard input queue */
   private Deque<Integer> keyBuffer = new LinkedList<>();

   /**
    * This method is called when the remote peer open a web socket connection.
    *
    * @param   session
    *          An active link of communications with a Remote WebSocket Endpoint.
    *
    */
   @OnWebSocketConnect
   public void onConnect(Session session)
   {
      this.remote = session.getRemote();
      sync();
   }

   /**
    * This method is called when a text message is received.
    *
    * @param   message
    *          The text message.
    */
   @OnWebSocketMessage
   public void onMessage(String message) {
      // TODO implements text protocol.
      try
      {
         remote.sendString("Salut Marius Gligor...");
      }
      catch (IOException e)
      {
         // TODO Auto-generated catch block
         e.printStackTrace();
      }
   }

   /**
    * This method is called when a binary message is received.
    *
    * @param   message
    *          Message content as an byte array.
    * @param   offset
    *          Message offset..
    * @param   length
    *          Message size.
    */
   @OnWebSocketMessage
   public void onMessage(byte[] message, int offset, int length)
   {
      // TODO implements binary protocol.
   }

   /**
    * This method is called when the connection is closed.
    *
    * @param   statusCode
    *          The status integer code.
    * @param   reason
    *          A string representing the reason.
    */
   @OnWebSocketClose
   public void onClose(int statusCode, String reason)
   {
      remote = null;
      LOG.logp(Level.INFO,
               "ChuiWebSimulator.onClose",
               "",
               LogHelper.generate("Connection closed status=%d reason=%s", statusCode, reason));
   }

   /**
    * This method is called on communication errors.
    *
    * @param   session
    *          An active link of communications with a Remote WebSocket Endpoint.
    * @param   error
    *          A <code>Throwable</code> instance representing the error condition.
    */
   @OnWebSocketError
   public void onError(Session session, Throwable error)
   {
      session.close();
      remote = null;
      
      LOG.logp(Level.SEVERE,
               "ChuiWebSimulator.onError",
               "",
               LogHelper.generate("WebSocket communication error!"),
               error);      
   }
   
   /**
    * Send a binary message.
    * 
    * @param   message
    *          Binary message.
    * 
    * @throws  IOException
    *          If cannot send the message.
    */
   public void sendMessage(byte[] message) throws IOException
   {
      if (remote != null)
      {
         ByteBuffer data = ByteBuffer.wrap(message);
         remote.sendBytes(data);
      }
   }
   
   /**
    * Read the next key from the FIFO key input buffer.  
    *
    * @return   The next key in the buffer or -1 if buffer is empty.
    */
   public int readKey()
   {
      synchronized(keyBuffer)
      {
         return keyBuffer.isEmpty() ? -1 : keyBuffer.pollFirst();
      }
   }
   
   /**
    * The beep() should be implemented by sending a command on the remote end point.
    * The command is part of the communication protocol.
    *
    */
   public void beep()
   {
      // TODO send a beep command down to JS code
   }

   /**
    * The sync() should be implemented by sending the screen status to the remote.
    * end point as a refresh screen operation.
    * The command is part of the communication protocol.
    *
    */
   public void sync()
   {
      // TODO send screen status down to JS
   }     
}